--[==============================[
  =         Camera Class         =
--]==============================]


require "util.class"
require "util.vector2"
require "util.matrix"

class("Camera",
	function (self, v_pos, v_offset, angle, zoom)
		self.position = v_pos or Vector(0, 0) -- position relative to the origin
		self.offset = v_offset or Vector2(0, 0) -- additional offset away from the center.
		self.angle = angle or 0
		self.setZoom = function (self, _zoom)
			self.zoom = math.clamp(_zoom, TUNING.CAMERA.ZOOM_MIN, TUNING.CAMERA.ZOOM_MAX)
		end
		local temp_zoom = zoom or TUNING.CAMERA.ZOOM
		self:setZoom(temp_zoom)
		-- offset_center is used to place the center of rotation at the center of the screen
		local w_height, w_width = love.window.getDimensions()
		self.offset_center = Vector2(w_height/2, w_width/2)
		self.computeMatrix = function (self)

			local m_rot = Matrix{ -- Rotation
				{math.cos(self.angle), math.sin(self.angle), 0},
				{-math.sin(self.angle), math.cos(self.angle), 0},
				{0, 0, 1}
			}

			local m_pos = Matrix{ -- World-space Position
				{1, 0, self.position.x},
				{0, 1, self.position.y},
				{0, 0, 1}
			}

			local m_zoom = Matrix{ -- Zoom
				{self.zoom, 0, 0},
				{0, self.zoom, 0, 0},
				{0, 0, 1}
			}

			local m_offset = Matrix{ -- Screen-space Offset
				{1, 0, self.offset_center.x + self.offset.x},
				{0, 1, self.offset_center.y + self.offset.y},
				{0, 0, 1}
			}

			self.matrix = nil

			-- reverse order than when applying them 1 by 1
			self.matrix =  m_offset * m_rot * m_zoom * m_pos
		end
		self:computeMatrix()
	end
)




--[=[
--[==============================[
  =         Camera Class         =
--]==============================]


require "util.class"
require "util.vector2"
require "util.matrix"

class("Camera",
	function (self, v_pos, v_offset, angle, zoom)
		local w_height, w_width = love.window.getDimensions()
		self.offset_center = Vector2(w_height/2, w_width/2)
		self.position = v_pos or Vector(0, 0)
		self.offset = v_offset or Vector2(0, 0)
		self.angle = angle or 0
		self.setZoom = function (self, _zoom)
			self.zoom = math.clamp(_zoom, TUNING.CAMERA.ZOOM_MIN, TUNING.CAMERA.ZOOM_MAX)
		end
		local temp_zoom = zoom or TUNING.CAMERA.ZOOM
		self:setZoom(temp_zoom)
		self.computeMatrix = function (self)

			local m_rot = Matrix{ -- Rotation
				{math.cos(self.angle), math.sin(self.angle), 0},
				{-math.sin(self.angle), math.cos(self.angle), 0},
				{0, 0, 1}
			}
			-- print_r(m_rot,"m_rot")

			local m_pos = Matrix{ -- World-space Offset
				{1, 0, self.position.x},
				{0, 1, self.position.y},
				{0, 0, 1}
			}
			-- print_r(m_pos,"m_pos")

			local m_zoom = Matrix{ -- Zoom
				{self.zoom, 0, 0},
				{0, self.zoom, 0, 0},
				{0, 0, 1}
			}
			-- print_r(m_zoom,"m_zoom")

			local m_offset = Matrix{ -- Screen-space Offset
				{1, 0, self.offset_center.x},
				{0, 1, self.offset_center.y},
				{0, 0, 1}
			}
			-- print_r(m_offset,"m_offset")
			self.matrix = nil
			self.matrix = m_zoom * m_rot * m_pos * m_offset -- reverse order than when applying them 1 by 1
		end
		self:computeMatrix()
	end
)

]=]



